/* Copyright (c) 2015, Electric Cloud, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* (As an historical note, the above license instantiates
 * the "BSD 2-Clause License" template that was presented
 * by the Open Source Initiative at
 *
 *   http://www.opensource.org/licenses/bsd-license.php
 *
 * on 2012-02-23.)
 */

/* Summary:
 *
 * Defines vrpsx_system_main_t; see its comments.
 */

#ifndef VRPSX_SYSTEM_MAIN_H
#define VRPSX_SYSTEM_MAIN_H 1

#include "vrpsx_system_initial.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/* The type of a main-like function that uses vrpsx_system_t.
 *
 * The caller promises that (initial != NULL) and that the members of
 * (*initial) are not NULL unless and until the called function modifies them.
 *
 * The called function may take ownership of arguments, environment, and
 * or system interface according to the rules of vrpsx_system_initial_t.
 * However, the lifetime of the vrpsx_system_initial_t instance itself
 * is controlled by the caller.
 *
 * This function type and its comments constitute an identified interface:
 * the clients are the function definitions and the implementations are
 * the callers.  A library may list such an implementation by using the
 * interface defined by "vrpsx_identified_function.h".
 */
typedef int vrpsx_system_main_t(
    vrpsx_system_initial_t *initial);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* VRPSX_SYSTEM_MAIN_H */
